'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _SubsonicFetch = require('../api/SubsonicFetch.js');

var _RadioMaker = require('../api/RadioMaker.js');

var _utils = require('../../utils/utils.js');

window.SubsonicCache = _SubsonicFetch.SubsonicCache; /* eslint no-unused-vars: 0 */

var SubsonicSource = {
  initialized: false,
  fetcher: _SubsonicFetch.SubsonicFetch,
  cache: _SubsonicFetch.SubsonicCache,

  login: function login(credentials) {
    var f = this.fetcher;
    this.cache.reset();
    this.initialized = true;
    return f.open(credentials.server, credentials.username, credentials.password, '0', true);
  },

  isLoginNeeded: function isLoginNeeded(credentials) {
    var f = this.fetcher;
    var c = credentials;
    if (!f) return true;
    return !f || f._u !== c.username || f._p !== c.password || f._s !== c.server || f._b !== c.bitrate;
  },

  refreshPlaylists: function refreshPlaylists() {
    var f = this.fetcher;
    return f.getPlaylists().then(function (res) {
      var p = res.playlists.playlist || [];
      var playlists = p.filter(function (pl) {
        pl.comment = pl.comment || '';
        return pl.comment.indexOf('{"radiostation"') !== 0 && !pl.name.startsWith('zz-sbr');
      });
      var stations = p.filter(function (pl) {
        pl.comment = pl.comment || '';
        return pl.comment.indexOf('{"radiostation"') === 0;
      });
      // process stations
      stations.forEach(function (station) {
        station.stationData = JSON.parse(station.comment);
        Object.setPrototypeOf(station.stationData, _RadioMaker.StationDataPrototype);
      });
      return {
        allPlaylists: p,
        playlists: playlists,
        stations: stations
      };
    }).catch(function (err) {
      console.log('err', err);
      return {
        allPlaylists: [],
        playlists: [],
        stations: []
      };
    });
  },

  refreshInternetStations: function refreshInternetStations() {
    var f = this.fetcher;
    return f.getInternetRadioStations().then(function (res) {
      return res.internetRadioStations.internetRadioStation || [];
    }).catch(function (err) {
      console.log('err', err);
      return [];
    });
  },


  fetchPlaylist: function fetchPlaylist(id) {
    var f = this.fetcher;
    return f.getPlaylist(id).then(function (res) {
      return {
        playlist: res.playlist,
        entries: res.playlist ? res.playlist.entry || [] : []
      };
    }).catch(function (err) {
      console.log('err', err);
      return {
        playlist: null,
        entries: []
      };
    });
  },

  fetchPlaylistAsAlbums: function fetchPlaylistAsAlbums(pl, offset, limit, folder, id3) {
    var f = this.fetcher;
    var prop = id3 ? 'albumList2' : 'albumList';
    return f.getPlaylistAsAlbumList(pl.id, id3).then(function (res) {
      return res[prop].album.slice(offset, offset + limit);
    }).catch(function (err) {
      console.log('err', err);
      return [];
    });
  },

  fetchVideos: function fetchVideos() {
    var f = this.fetcher;
    return f.getVideos().then(function (res) {
      return res.videos && res.videos.video ? res.videos.video : [];
    }).catch(function (err) {
      console.log('err', err);
      return [];
    });
  },

  fetchSong: function fetchSong(id) {
    var f = this.fetcher;
    return f.getSong(id).then(function (res) {
      console.log('song', res);
      return res.song;
    }).catch(function (err) {
      console.log('err', err);
      return null;
    });
  },

  getNowPlaying: function getNowPlaying(id) {
    var f = this.fetcher;
    return f.getNowPlaying().then(function (res) {
      console.log('nowPlaying', res);
      return res.nowPlaying;
    }).catch(function (err) {
      console.log('err', err);
      return [];
    });
  },

  fetchAlbums: function fetchAlbums(query, offset, limit, folder, id3) {
    var f = this.fetcher;
    if (!f) return;
    var method = id3 ? 'getAlbumList2' : 'getAlbumList';
    var params = {
      size: limit,
      offset: offset
    };
    if (folder && folder * 1 !== -1) {
      params.musicFolderId = folder;
    }
    if (query.indexOf('byYear') === 0) {
      if (query === 'byYear reversed') {
        query = 'byYear';
        params.fromYear = 2100;
        params.toYear = 1500;
      } else {
        params.fromYear = 1500;
        params.toYear = 2100;
      }
    }
    return f[method](query, params).then(function (res) {
      var albumList = res.albumList || res.albumList2;
      return {
        albumList: albumList ? albumList.album || [] : [],
        offset: offset
      };
    }).catch(function (err) {
      console.log('err', err);
      return {
        albumList: [],
        offset: offset
      };
    });
  },

  fetchAllArtists: function fetchAllArtists() {
    var f = this.fetcher;
    return f.getArtists().then(function (res) {
      return res.artists.index || [];
    }).catch(function (err) {
      console.log('err', err);
      return [];
    });
  },

  fetchArtists: function fetchArtists(musicFolderId) {
    var f = this.fetcher;
    if (musicFolderId === -1) musicFolderId = undefined;
    return f.getArtists(musicFolderId).then(function (res) {
      return {
        indexes: res.artists.index,
        musicFolderId: musicFolderId
      };
    }).catch(function (err) {
      console.log('err', err);
      return {
        albumList: [],
        musicFolderId: musicFolderId
      };
    });
  },

  fetchArtist: function fetchArtist(id) {
    var f = this.fetcher;
    // sorting albums by year
    return f.getArtist(id).then(function (res) {
      res.artist.album.sort(function (a, b) {
        var da = a.year || 0,
            db = b.year || 0;
        var ta = a.name || 0,
            tb = b.name || 0;
        if (da === db) {
          if (ta === tb) return a.path < b.path ? -1 : a.path > b.path ? 1 : 0;
          return ta - tb;
        } else {
          return da - db;
        }
      });
      return res.artist;
    }).catch(function (err) {
      console.log('err', err);
      return null;
    });
  },

  fetchAlbum: function fetchAlbum(id) {
    var f = this.fetcher;
    return f.getAlbum(id).then(function (res) {
      return res.album;
    }).catch(function (err) {
      console.log('err', err);
      return null;
    });
  },

  fetchMusicFolders: function fetchMusicFolders() {
    var f = this.fetcher;
    return f.getMusicFolders().then(function (res) {
      return res.musicFolders ? res.musicFolders.musicFolder || [] : [];
    }).catch(function (err) {
      console.log('err', err);
      return [];
    });
  },

  fetchIndexes: function fetchIndexes(musicFolderId) {
    var f = this.fetcher;
    if (musicFolderId === -1) musicFolderId = undefined;
    return f.getIndexes(musicFolderId).then(function (res) {
      return {
        indexes: res.indexes.index,
        musicFolderId: musicFolderId
      };
    }).catch(function (err) {
      console.log('err', err);
      return {
        indexes: [],
        musicFolderId: musicFolderId
      };
    });
  },

  fetchRandomSongs: function fetchRandomSongs(musicFolderId) {
    var f = this.fetcher;
    if (musicFolderId === -1) musicFolderId = null;
    return f.getRandomSongs(null, null, null, musicFolderId, 100).then(function (res) {
      return res.randomSongs.song || [];
    }).catch(function (err) {
      console.log('err', err);
      return [];
    });
  },

  fetchMusicDirectory: function fetchMusicDirectory(id) {
    var _this = this;

    var f = this.fetcher;
    if (this.cache.MusicPaths[id]) {
      return Promise.resolve({
        directory: this.cache.MusicPaths[id],
        children: this.cache.MusicPaths[id].child
      });
    }
    return f.getMusicDirectory(id).then(function (res) {
      _this.cache.MusicPaths[id] = res.directory;
      return {
        directory: res.directory,
        children: res.directory.child
      };
    }).catch(function (err) {
      console.log('err', err);
      return {
        directory: null,
        children: []
      };
    });
  },

  fetchPodcasts: function fetchPodcasts() {
    var f = this.fetcher;
    return f.getPodcasts().then(function (res) {
      return res.podcasts.channel;
    }).catch(function (err) {
      console.log('err', err);
      return [];
    });
  },

  fetchPlayQueue: function fetchPlayQueue() {
    var f = this.fetcher;
    return f.getPlayQueue().then(function (res) {
      res.playQueue = res.playQueue || {};
      return res.playQueue ? res.playQueue : {
        entry: [],
        current: 0,
        position: 0
      };
    }).catch(function (err) {
      console.log('err', err);
      return {
        entry: [],
        current: 0,
        position: 0
      };
    });
  },

  savePlayQueue: function savePlayQueue(queue, idx, currentMediaTime) {
    idx = queue[idx].id;
    queue = queue.map(function (s) {
      return s.id;
    });
    var f = this.fetcher;
    return f.savePlayQueue(queue, idx, currentMediaTime);
  },

  getCoverArtURL: function getCoverArtURL(id, size) {
    var f = this.fetcher;
    if ((!id || id === '-1' || id === -1) && f.serverAPIVersion === '1.11.0') {
      return './logos/genericcoverart.jpeg';
    }
    return f.getCoverArtURL(id, size);
  },

  getDownloadURL: function getDownloadURL(id) {
    var f = this.fetcher;
    return f.getDownloadURL(id);
  },

  getStreamingURL: function getStreamingURL(id) {
    var f = this.fetcher;
    return f.getStreamingURL(id);
  },

  getBookmarks: function getBookmarks() {
    var f = this.fetcher;
    return f.getBookmarks().then(function (res) {
      return res.bookmarks ? res.bookmarks.bookmark : null;
    });
  },

  createBookmark: function createBookmark(id, position, comment) {
    var f = this.fetcher;
    return f.createBookmark(id, position, comment);
  },

  deleteBookmark: function deleteBookmark(id) {
    var f = this.fetcher;
    return f.deleteBookmark(id);
  },

  search: function search(query, id3, params) {
    var f = this.fetcher;
    return f.search(query, id3, params).then(function (res) {
      return res.searchResult2 || res.searchResult3;
    }).catch(function (err) {
      console.error('err');
      return null;
    });
  },

  generateStation: function generateStation(station) {
    var stationVM = Object.create(_RadioMaker.StationVM);
    stationVM.s = station;
    stationVM.generateOnly = true;
    return new Promise(stationVM.generate.bind(stationVM)).then(function (data) {
      var r = {
        playlist: {
          id: 'xx',
          name: station.name,
          entry: data,
          songCount: data.length
        }
      };
      return r;
    }).catch(function (err) {
      console.error(err);
      return null;
    });
  },

  playMusicDirectorySongs: function playMusicDirectorySongs(musicDirectory, type) {
    // console.log('playMusicDirectorySongs', musicDirectory, type);
    var f = this.fetcher;
    if (type === 'top') {
      return f.getTopSongs(musicDirectory.name, 50).then(function (res) {
        return res.topSongs.song || [];
      }).catch(function (err) {
        console.log('err', err);
        return [];
      });
    }
    if (type === 'radio') {
      return f.getSimilarSongs(musicDirectory.id, false, 100).then(function (res) {
        return res.similarSongs.song || [];
      }).catch(function (err) {
        console.log('err', err);
        return [];
      });
    }
    if (type === 'shuffleAlbums') {
      return f.getMusicDirectoryAlbums(musicDirectory.id).then(function (res) {
        var v = res.musicDirectoryAlbums.album;
        (0, _utils.arrayShuffle)(v);
        (0, _utils.arrayShuffle)(v);
        (0, _utils.arrayShuffle)(v);
        (0, _utils.arrayShuffle)(v);
        (0, _utils.arrayShuffle)(v);
        var songs = v.flatten();
        return songs || [];
      }).catch(function (err) {
        console.log('err', err);
        return [];
      });
    }
    return f.getMusicDirectorySongs(musicDirectory.id).then(function (res) {
      // console.log(res);
      if (type === 'shuffle') {
        return (0, _utils.arrayShuffle)(res.musicDirectory.song || []);
      }
      return res.musicDirectory.song || [];
    }).catch(function (err) {
      console.log('err', err);
      return [];
    });
  },
  playArtistSongs: function playArtistSongs(artist, type) {
    var f = this.fetcher;
    if (type === 'top') {
      return f.getTopSongs(artist.name, 50).then(function (res) {
        return res.topSongs.song || [];
      }).catch(function (err) {
        console.log('err', err);
        return [];
      });
    } else if (type === 'radio') {
      return f.getSimilarSongs2(artist.id, 100).then(function (res) {
        return res.similarSongs2.song || [];
      }).catch(function (err) {
        console.log('err', err);
        return [];
      });
    }
    if (type === 'shuffleAlbums') {
      return f.getArtistAlbums(artist.id).then(function (res) {
        var v = res.albumCollection.albums;
        (0, _utils.arrayShuffle)(v);
        (0, _utils.arrayShuffle)(v);
        (0, _utils.arrayShuffle)(v);
        (0, _utils.arrayShuffle)(v);
        (0, _utils.arrayShuffle)(v);
        var songs = v.flatten();
        return songs || [];
      }).catch(function (err) {
        console.log('err', err);
        return [];
      });
    }
    return f.getArtistSongs(artist.id).then(function (res) {
      res.albumCollection.song = res.albumCollection.song || [];
      if (type === 'shuffle') {
        res.albumCollection.song = (0, _utils.arrayShuffle)(res.albumCollection.song);
      } else {
        res.albumCollection.song = res.albumCollection.song.sort(function (s1, s2) {
          s1.year = s1.year || 0;
          s2.year = s2.year || 0;
          s1.track = s1.track || 1;
          s2.track = s2.track || 1;
          s1.discNumber = s1.discNumber || 1;
          s2.discNumber = s2.discNumber || 1;
          if (s1.year && s2.year && s1.year !== s2.year) return s1.year - s2.year;
          if (s1.album !== s2.album) return s1.album < s2.album ? -1 : 1;
          if (s1.discNumber !== s2.discNumber) return s1.discNumber - s2.discNumber;
          if (s1.track !== s2.track) return s1.track - s2.track;
          return s1.path < s2.path ? -1 : 1;
        });
      }
      return res.albumCollection.song;
    }).catch(function (err) {
      console.log('err', err);
      return [];
    });
  }
};

window.SubsonicSource = SubsonicSource;

exports.default = SubsonicSource;
module.exports = exports.default;