/**
 * PlayerHandler
 *
 * This is a handler through which the application will interact
 * with both the RemotePlayer and LocalPlayer. Combining these two into
 * one interface is one approach to the dual-player nature of a Cast
 * Chrome application. Otherwise, the state of the RemotePlayer can be
 * queried at any time to decide whether to interact with the local
 * or remote players.
 *
 * To set the player used, implement the following methods for a target object
 * and call setTarget(target).
 *
 * Methods to implement:
 *  - play()
 *  - pause()
 *  - stop()
 *  - seekTo(time)
 *  - load(mediaIndex)
 *  - getMediaDuration()
 *  - getCurrentMediaTime()
 *  - mute()
 *  - unMute()
 *  - isMuted()
 */

import { toHHMMSS } from '../utils/utils.js';
import { PLAYER_STATE } from './Constants';

var PlayerHandler = function(castPlayer) {
  this.target = {};

  this.setTarget = function(target) {
    this.target = target;
  };

  this.play = function() {
    console.warn('ph play', castPlayer.playerState, castPlayer.autoplay);
    if (castPlayer.playerState === PLAYER_STATE.PLAYING) return;

    castPlayer.autoplay = true;
    if (
      castPlayer.playerState !== PLAYER_STATE.PLAYING &&
      // castPlayer.playerState !== PLAYER_STATE.STOPPED &&
      castPlayer.playerState !== PLAYER_STATE.PAUSED &&
      castPlayer.playerState !== PLAYER_STATE.LOADED
    ) {
      this.load(castPlayer.currentMedia);
      return;
    }

    this.target.play();
    castPlayer.playerState = PLAYER_STATE.PLAYING;
    castPlayer.startProgressTimer();
  };

  this.pause = function(autoplay) {
    // console.warn('ph pause', castPlayer.playerState);
    castPlayer.autoplay =
      autoplay ||
      castPlayer.playerState === PLAYER_STATE.IDLE ||
      castPlayer.playerState === PLAYER_STATE.STOPPED;

    if (castPlayer.playerState !== PLAYER_STATE.PLAYING) {
      return;
    }

    this.target.pause();
    castPlayer.playerState = PLAYER_STATE.PAUSED;
  };

  this.playOrPause = function() {
    // console.log('ph playOrPause');
    if (castPlayer.playerState === PLAYER_STATE.PLAYING) {
      castPlayer.autoplay = false;
      castPlayer.playerHandler.pause();
    } else {
      castPlayer.autoplay = true;
      castPlayer.playerHandler.play();
    }
  };

  this.isPlaying = function() {
    return castPlayer.playerState === PLAYER_STATE.PLAYING;
  };

  this.isPaused = function() {
    return castPlayer.playerState !== PLAYER_STATE.PLAYING;
  };

  this.stop = function() {
    // internal control, does NOT change autoplay state as opposed to explicit play/pause action from button
    var currentAutoPlay = castPlayer.autoplay;
    castPlayer.playerHandler.pause(currentAutoPlay);
    castPlayer.autoplay = currentAutoPlay;
    castPlayer.playerState = PLAYER_STATE.STOPPED;
  };

  this.load = function(media, initialTime) {
    castPlayer.playerState = PLAYER_STATE.LOADING;
    castPlayer.currentMediaTime = initialTime || 0;
    castPlayer.currentMedia = media;
    // I have no idea why, but this dispatch sends the WRONG status - it sends the last one sent, but only in this one case
    // castPlayer.updateStatusListeners('load');
    this.target.load(media);
  };

  this.loaded = function() {
    // console.warn('loaded', castPlayer.autoplay);
    castPlayer.currentMediaDuration = this.getMediaDuration();
    castPlayer.playerState = PLAYER_STATE.LOADED;

    // console.warn('loaded switchTime', castPlayer.switchTimeRestore);

    castPlayer.currentMediaTime =
      castPlayer.switchTimeRestore || castPlayer.currentMediaTime;
    castPlayer.switchTimeRestore = null;
    if (castPlayer.currentMediaTime > 0) {
      // console.warn('seekTo', castPlayer.currentMediaTime);
      this.seekTo(castPlayer.currentMediaTime);
    }
    if (castPlayer.autoplay) {
      // console.warn('autoplay');
      this.play();
      // castPlayer.startProgressTimer();
    }
    castPlayer.updateStatusListeners('loaded');
  };

  this.getCurrentMediaTime = function() {
    return this.target.getCurrentMediaTime();
  };

  this.getMediaDuration = function() {
    return this.target.getMediaDuration();
  };

  this.mute = function() {
    this.target.mute();
  };

  this.unMute = function() {
    this.target.unMute();
  };

  this.isMuted = function() {
    return this.target.isMuted();
  };

  this.seekTo = function(time) {
    this.target.seekTo(time);
    if (castPlayer.playerState !== PLAYER_STATE.PLAYING) {
      castPlayer.updateStatusListeners('seeking');
    }
  };

  this.getState = function(reason) {
    return Object.assign(
      {},
      {
        reason: reason,
        isPlaying: castPlayer.playerState === PLAYER_STATE.PLAYING,
        isPaused: castPlayer.playerState !== PLAYER_STATE.PLAYING,
        isIdle: castPlayer.playerState === PLAYER_STATE.IDLE,
        isStopped: castPlayer.playerState === PLAYER_STATE.STOPPED,
        isLoading:
          castPlayer.playerState === PLAYER_STATE.LOADING ||
          !castPlayer.currentMedia,
        duration: this.target.getMediaDuration(),
        durationString: toHHMMSS(this.target.getMediaDuration()),
        currentMediaTime: this.target.getCurrentMediaTime(),
        currentMediaTimeString: toHHMMSS(this.target.getCurrentMediaTime()),
        isMuted: this.target.isMuted(),
        media: castPlayer.currentMedia,
        autoplay: castPlayer.autoplay,
        actions: this
      }
    );
  };
};

export default PlayerHandler;
