import React, { Component } from 'react';
import { Redirect, Link } from 'react-router-dom';

import Typography from '@material-ui/core/Typography';
import GridList from '@material-ui/core/GridList';
import IconButton from '@material-ui/core/IconButton';
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import ListItemSecondaryAction from '@material-ui/core/ListItemSecondaryAction';
import FolderOpen from '@material-ui/icons/FolderOpen';
import { withStyles } from '@material-ui/core/styles';
import VideoLibrary from '@material-ui/icons/VideoLibrary';

import ArtistActionButtons from './ArtistActionButtons';

import SubsonicActions from 'subsonic/actions/SubsonicActions';

import CoverGridTile from 'subsonic/components/styled/CoverGridTile';
import ScrollToTopFab from 'subsonic/components/styled/ScrollToTopFab';
import SongList from 'subsonic/components/browsers/SongList';

const styles = theme => ({
  li: {
    listStyle: 'none'
  }
});

class DirectoryBrowser extends Component {
  constructor(props) {
    super(props);
    this.myRef = React.createRef();
  }

  playItem = song => {
    if (this.props.remotes.remoteState === 'sender') {
      this.props.history.push('/loading/song/' + song.id);
    } else {
      SubsonicActions.startPlaying(song, 0);
      this.props.history.push('/player');
    }
  };

  loadAlbumFolder = album => {
    this.props.history.push('/loading/directory/' + album.id);
  };

  playFolder = album => {
    this.props.history.push('/loading/directoryAction/' + album.id + '/all');
  };

  updateDimensions = () => {
    this.setState({ width: window.innerWidth, height: window.innerHeight });
  };

  componentDidMount = () => {
    window.addEventListener('resize', this.updateDimensions);
    this.updateDimensions();
  };

  componentWillUnmount = () => {
    window.removeEventListener('resize', this.updateDimensions);
  };

  render() {
    const {
      subsonic,
      SubsonicSource,
      match,
      classes,
      showBackButton,
      history
    } = this.props;
    const isGrid = subsonic.albumsAsGrid;
    const paramsId = match.params.id;
    if (!subsonic || !subsonic.isConnected) {
      return <Typography>awaiting login...</Typography>;
    }

    if (!subsonic.musicDirectory) {
      if (paramsId === undefined || paramsId === 'undefined') {
        return <Typography>problem 1</Typography>;
      }
      return <Redirect to={'/loading/directory/' + paramsId} />;
    }

    const { id } = subsonic.musicDirectory;
    if (paramsId === undefined || paramsId === 'undefined') {
      return <Typography>problem 1</Typography>;
    }
    if ('' + paramsId !== '' + id) {
      return <Redirect to={'/loading/directory/' + paramsId} />;
    }
    const contents = subsonic.musicDirectoryContents;
    if (!contents) {
      return <Typography>oops?</Typography>;
    }
    const sortedContentsInit = {
      folders: [],
      albums: [],
      songs: []
    };
    const sortedContents = contents.reduce((sortedContents, v, i, array) => {
      if (v.isDir && v.coverArt) sortedContents.albums.push(v);
      else if (v.isDir) sortedContents.folders.push(v);
      else sortedContents.songs.push(v);
      return sortedContents;
    }, sortedContentsInit);
    // console.log(sortedContents);

    // add folder icon
    const folderSection = sortedContents.folders.map(item => (
      <ListItem
        classes={{ container: classes.li }}
        key={`item-${item.id}`}
        button
        component={Link}
        to={`/loading/directory/${item.id}`}
      >
        <ListItemIcon>
          <FolderOpen />
        </ListItemIcon>

        <ListItemText primary={`${item.title}`} />
        <ListItemSecondaryAction>
          <IconButton
            aria-label="Play"
            onClick={this.playFolder.bind(this, item)}
          >
            <VideoLibrary />
          </IconButton>
        </ListItemSecondaryAction>
      </ListItem>
    ));

    // convert to grid
    let innerWidth = window.innerWidth;
    if (this.myRef && this.myRef.current) {
      innerWidth = this.myRef.current.offsetWidth;
    }

    if (isNaN(innerWidth)) innerWidth = 360;
    const spacing = innerWidth > 360 ? 10 : 6;
    const cellSize = innerWidth > 360 ? 170 : 164;

    const cols = innerWidth / (cellSize + spacing);
    const width = cols * (cellSize + spacing);

    const PlayLink = React.forwardRef((props, ref) => {
      const link = '/loading/directoryAction/' + props.id + '/all';
      return (
        <Link innerRef={ref} to={link}>
          {props.children}
        </Link>
      );
    });

    const albumSection = isGrid ? (
      <div ref={this.myRef}>
        <GridList
          cellHeight={cellSize}
          width={width}
          cols={cols}
          spacing={spacing}
        >
          {sortedContents.albums.map((item, idx) => (
            <CoverGridTile
              onClick={this.loadAlbumFolder.bind(this, item)}
              key={'alb' + item.id}
              subname={item.artist}
              name={item.title || item.album || item.name}
              normal
              size={160}
              coverArt={item.coverArt}
              linkComponent={PlayLink}
              id={item.id}
              SubsonicSource={SubsonicSource}
            />
          ))}
        </GridList>
      </div>
    ) : (
      <List>
        {sortedContents.albums.map(item => (
          <ListItem
            key={'alb' + item.id}
            button
            component={Link}
            to={`/loading/directory/${item.id}`}
          >
            <ListItemIcon>
              <img
                width={48}
                height={48}
                alt=""
                src={SubsonicSource.getCoverArtURL(item.coverArt, 48)}
              />
            </ListItemIcon>
            <ListItemText
              primary={item.title || item.album || item.name}
              secondary={item.artist}
            />
            <ListItemSecondaryAction>
              <IconButton
                aria-label="Play"
                onClick={this.playFolder.bind(this, item)}
              >
                <VideoLibrary />
              </IconButton>
            </ListItemSecondaryAction>
          </ListItem>
        ))}
      </List>
    );

    // render first one's cover art, then list the rest
    // const songSection = sortedContents.songs.map(item => (
    //   <ListItem
    //     key={`item-${item.id}`}
    //     button
    //     onClick={this.playItem.bind(this, item)}
    //   >
    //     <ListItemIcon>
    //       <img src={SubsonicSource.getCoverArtURL(item.coverArt, 24)} />
    //     </ListItemIcon>
    //     <ListItemText primary={`${item.title}`} />
    //   </ListItem>
    // ));

    const songSection = (
      <SongList
        contents={sortedContents.songs}
        SubsonicSource={SubsonicSource}
        showMeta={false}
        handleSongClick={this.playItem}
      />
    );

    return (
      <div>
        <ArtistActionButtons
          id={id}
          loadbase="directoryAction"
          SubsonicActions={SubsonicActions}
          isGrid={subsonic.albumsAsGrid}
          showBackButton={showBackButton}
          history={history}
        />
        <List>{folderSection}</List>
        {albumSection}
        {songSection}
        <ScrollToTopFab {...this.props.fabProps} />
      </div>
    );
  }
}

export default withStyles(styles)(DirectoryBrowser);
